from herschel.pacs.spg.spec.beams import ExtendedSpectralSourceAdapter
from math import *


## A Gaussian source with a Gaussian emission line + polynomial continuum
## User-defined parameters are the FWHM of the Gaussian, its offset from the 
## centre of the field (the centre of the field will be set when the model 
## is used in the task pacsSpecFromModel, and is the centre of the field 
## of the cube you are modelling), the flux values of the line and continuum 
##
class MyGaussSpectral(ExtendedSpectralSourceAdapter):
    fwhm = 0.0
    offRa = 0.0
    offDec = 0.0
    contFlux = 0.0
    contSlope = 0.0
    lineAmplitude = 0.0
    lineCenter = 0.0
    lineWidth = 0.0

    def __init__(self,fwhm,offRa,offDec,contFlux,contSlope,lineAmplitude,lineCenter,lineWidth):
        self.sourceSize = fwhm/2.3548
        self.offRa = offRa
        self.offDec = offDec
        self.contFlux = contFlux
        self.contSlope = contSlope
        self.lineAmplitude = lineAmplitude
        self.lineCenter = lineCenter
        self.lineWidth = lineWidth

    # Required
    def getFlux(self, relRa, relDec, wavelength):
        r = sqrt((relRa-self.offRa)**2 + (relDec-self.offDec)**2)
        spectrum = self.contFlux + self.contSlope * (wavelength-self.lineCenter)
        spectrum +=  self.lineAmplitude * exp(-0.5 * ((wavelength-self.lineCenter) / self.lineWidth)**2)
        flux = spectrum * exp(-0.5 * (r / self.sourceSize)**2)
        return flux

    def getFluxImage(self,relRa,relDec, wavelength):
        r = sqrt((relRa-self.offRa)**2 + (relDec-self.offDec)**2)
        spectrum = self.contFlux + self.contSlope * (wavelength-self.lineCenter)
        spectrum += self.lineAmplitude * exp(-0.5 * ((wavelength-self.lineCenter) / self.lineWidth)**2)
        self.fluxImage = spectrum * exp(-0.5 * (r / self.sourceSize)**2)
        return self.fluxImage

    # Rendering of layer on the beam grid - optional
    def renderGaussLayer(self, size=109, dpix=0.5, wavelength=66.38):
        self.image=Double2d(size, size)
        for y in range( size ):
           for x in range( size ):
             xim = (-dpix)*(x-(size/2))
             yim = (dpix)*(y-(size/2))
             self.image.set(y,x, self.getFluxImage(xim, yim, wavelength))
        return self.image
    
    # Rendering of layer image on the beam grid - optional
    def getSourceModelImage(self, size=109, dpix=0.5, wavelength=66.38):
        return self.renderGaussLayer(size, dpix, wavelength)

## Model visualisation 
def imageModelInPacsSpecFromModel(sourceModel,naxis, pixAngSize, wave):
   sourceModelImage_01 = SimpleImage(image=sourceModel.getSourceModelImage(naxis*5, pixAngSize/5, wave))
   res = Double2d(naxis,naxis)
   for i in range(naxis):
      for j in range(naxis):
         flux = 0.0
         for p in range(5):
            for q in range(5):
               i2=i*5+p
               j2=j*5+q
               flux += sourceModelImage_01.image[j2,i2]
         res[j,i] = flux/25.
   sourceModelImage_05 = SimpleImage(image=res)
   return sourceModelImage_05

## A completely flat source 
class MyFlatSourceSpectral(ExtendedSpectralSourceAdapter):

    def __init__(self,contFlux,contSlope,lineCenter):
        self.contFlux = contFlux
        self.contSlope = contSlope
        self.lineCenter = lineCenter

    def getFlux(self, relRa, relDec, wavelength):
        spectrum = self.contFlux + self.contSlope * (wavelength-self.lineCenter)
        return spectrum

    def getFluxImage(self,relRa,relDec, wavelength):
        spectrum = self.contFlux + self.contSlope * (wavelength-self.lineCenter)
        self.fluxImage = spectrum 
        return self.fluxImage

    def renderGaussLayer(self, size=109, dpix=0.5, wavelength=79.0):
        self.image=Double2d(size, size)
        for y in range( size ):
           for x in range( size ):
             xim = (-dpix)*(x-(size/2))
             yim = (dpix)*(y-(size/2))
             self.image.set(y,x, self.getFluxImage(xim, yim, wavelength))
        return self.image
    
    def getSourceModelImage(self, size=109, dpix=0.5, wavelength=79.0):
        return self.renderGaussLayer(size, dpix, wavelength)

